@@ -8,7 +8,7 @@
  * The CHOLMOD/Modify Module is licensed under Version 2.0 of the GNU
  * General Public License.  See gpl.txt for a text of the license.
  * CHOLMOD is also available under other licenses; contact authors for details.
- * http://www.cise.ufl.edu/research/sparse
+ * http://www.suitesparse.com
  * -------------------------------------------------------------------------- */
 
 /* Updates/downdates the LDL' factorization (symbolic, then numeric), by
@@ -519,7 +519,9 @@ int CHOLMOD(updown_mask)
     /* get inputs */
     /* ---------------------------------------------------------------------- */
 
-    mark = CHOLMOD(clear_flag) (Common) ;
+    /* mark = CHOLMOD(clear_flag) (Common) ; */
+    CHOLMOD_CLEAR_FLAG (Common) ;
+    mark = Common->mark ;
 
     PRINT1 (("updown, rank %g update %d\n", (double) C->ncol, update)) ;
     DEBUG (CHOLMOD(dump_factor) (L, "input L for updown", Common)) ;
